@@ -64,7 +64,8 @@ extern int update_battery_date;
 extern int debug_level;
 extern int rename_ups;
 extern int terminate_on_powerfail;
-extern int kill_ups_power;
+extern int hibernate_ups;
+extern int shutdown_ups;
 extern int dumb_mode_test;
 extern int go_background;
 
@@ -86,7 +87,8 @@ extern void make_pid(void);
 /* In apcdevice.c */
 void setup_device(UPSINFO *ups);
 extern void setup_serial(UPSINFO *ups);
-extern void kill_power(UPSINFO *ups);
+extern void initiate_hibernate(UPSINFO *ups);
+extern void initiate_shutdown(UPSINFO *ups);
 extern void save_dumb_status(UPSINFO *ups);
 extern int check_serial(UPSINFO *ups);
 extern void prep_device(UPSINFO *ups);
@@ -177,6 +179,7 @@ extern void do_reports(UPSINFO *ups);
 
 /* In apcsmart.c */
 extern int apc_enable(UPSINFO *ups);
+extern int writechar(char a, UPSINFO *ups);
 extern int getline(char *s, int len, UPSINFO *ups);
 extern void UPSlinkCheck(UPSINFO *ups);
 extern char *smart_poll(char cmd, UPSINFO *ups);
@@ -215,6 +218,7 @@ extern void wait_for_termination(int serial_pid);
 /* In apclog.c */
 extern void log_event(const UPSINFO *ups, int level, const char *fmt, ...);
 extern void logf(const char *fmt, ...);
+extern void hex_dump(int level, void *data, unsigned int len);
 
 /* In apcnetlib.c */
 extern int net_open(const char *host, char *service, int port);
