@@ -64,7 +64,7 @@ static void program_smart_eeprom(void);
 static void print_eeprom_values(UPSINFO *ups);
 static void smart_ttymode(void);
 static void parse_eeprom_cmds(char *eprom, char locale);
-int apcsmart_ups_program_eeprom(UPSINFO *ups, int command, char *data);
+int apcsmart_ups_program_eeprom(UPSINFO *ups, int command, const char *data);
 static void print_valid_eeprom_values(UPSINFO *ups);
 #endif
 
@@ -84,10 +84,11 @@ static void usb_set_battery_date(void);
 static void usb_get_manf_date(void);
 static void usb_set_alarm(void);
 static void usb_set_sens(void);
+static void usb_set_xferv(int lowhigh);
 #endif
 
 static void strip_trailing_junk(char *cmd);
-static char *get_cmd(char *prompt);
+static char *get_cmd(const char *prompt);
 static int write_file(char *buf);
 
 
@@ -103,7 +104,7 @@ static int test5_done = 0;
 int shm_OK = 0;
 
 /* Print a message, and also write it to an output file */
-static void pmsg(char *fmt, ...)
+static void pmsg(const char *fmt, ...)
 {
    char buf[3000];
    va_list arg_ptr;
@@ -452,13 +453,11 @@ int main(int argc, char *argv[])
    pmsg("Setting up the port ...\n");
    setup_device(ups);
 
-   if (kill_ups_power) {
+   if (hibernate_ups) {
       pmsg("apctest: bad option, I cannot do a killpower\n");
       apctest_terminate(0);
    }
 
-   make_pid_file();
-
    init_signals(apctest_terminate);
 
    if (ups->fd != -1) {
@@ -798,7 +797,7 @@ static void test6(void)
    ptime();
    print_bits(bits);
    make_file(ups, ups->pwrfailpath);
-   kill_power(ups);
+   initiate_hibernate(ups);
    unlink(ups->pwrfailpath);
    ptime();
 
@@ -1541,15 +1540,17 @@ static void do_usb_testing(void)
 
    while (!quit) {
       pmsg("\n"
-           "1) Test kill UPS power\n"
-           "2) Perform self-test\n"
-           "3) Read last self-test result\n"
-           "4) Change battery date\n"
-           "5) View battery date\n"
-           "6) View manufacturing date\n"
-           "7) Set alarm behavior\n"
-           "8) Set sensitivity\n"
-           "9) Quit\n\n");
+           "1)  Test kill UPS power\n"
+           "2)  Perform self-test\n"
+           "3)  Read last self-test result\n"
+           "4)  Change battery date\n"
+           "5)  View battery date\n"
+           "6)  View manufacturing date\n"
+           "7)  Set alarm behavior\n"
+           "8)  Set sensitivity\n"
+           "9)  Set low transfer voltage\n"
+           "10) Set high transfer voltage\n"
+           "11) Quit\n\n");
 
       cmd = get_cmd("Select function number: ");
       if (cmd) {
@@ -1581,14 +1582,20 @@ static void do_usb_testing(void)
             usb_set_sens();
             break;
          case 9:
+            usb_set_xferv(0);
+            break;
+         case 10:
+            usb_set_xferv(1);
+            break;
+         case 11:
             quit = TRUE;
             break;
          default:
-            pmsg("Illegal response. Please enter 1-9\n");
+            pmsg("Illegal response. Please enter 1-11\n");
             break;
          }
       } else {
-         pmsg("Illegal response. Please enter 1-9\n");
+         pmsg("Illegal response. Please enter 1-11\n");
       }
    }
    ptime();
@@ -1600,6 +1607,60 @@ static void do_usb_testing(void)
 
 #ifdef HAVE_USB_DRIVER
 
+static void usb_set_xferv(int lowhigh)
+{
+   int result;
+   char* cmd;
+   const char *text;
+   int ci;
+
+   if (lowhigh) {
+      text = "HIGH";
+      ci = CI_HTRANS;
+   } else {
+      text = "LOW";
+      ci = CI_LTRANS;
+   }
+
+   if (!usb_read_int_from_ups(ups, ci, &result)) {
+      pmsg("\nI don't know how to control the %s transfer voltage "
+         " settings on your UPS.\n", text);
+      return;
+   }
+
+   int newval;
+   do {
+      pmsg("Current %s transfer voltage setting: %d Volts\n", text, result);
+
+      pmsg("Enter new %s transfer voltage (0 to cancel): ", text);
+      cmd = get_cmd("");
+      newval = atoi(cmd);
+
+      // Check for exit
+      if (newval == 0)
+         return;
+
+      // Write new value
+      usb_write_int_to_ups(ups, ci, newval, text);
+
+      // Give write a chance to work, then read new value
+      sleep(1);
+      result = 0;
+      usb_read_int_from_ups(ups, ci, &result);
+
+      if (result != newval) {
+         pmsg("FAILED to set new %s transfer voltage.\n\n"
+              "This is probably because you entered a value that is out-of-range\n"
+              "for your UPS. The acceptable range of values varies based on UPS\n"
+              "model. The UPS has probably set the value as close as possible to\n"
+              "what you requested.\n\n", text);
+      }
+   }
+   while (result != newval);
+
+   pmsg("New %s transfer voltage setting: %d Volts\n", text, result);
+}
+
 static void usb_set_sens(void)
 {
    int result;
@@ -1678,7 +1739,6 @@ static void usb_set_sens(void)
       pmsg("UNKNOWN\n");
       break;
    }
-   
 }
 
 static void usb_set_alarm(void)
@@ -1766,7 +1826,7 @@ static void usb_kill_power_test(void)
    pmsg("calling kill_power function.\n");
 
    make_file(ups, ups->pwrfailpath);
-   kill_power(ups);
+   initiate_hibernate(ups);
    unlink(ups->pwrfailpath);
 
    ptime();
@@ -1980,7 +2040,7 @@ static void usb_get_manf_date(void)
 #endif
 
 /* Get next input command from the terminal */
-static char *get_cmd(char *prompt)
+static char *get_cmd(const char *prompt)
 {
    static char cmd[1000];
 
@@ -2033,8 +2093,8 @@ static UPSINFO eeprom_ups;
  */
 static struct {
    char cmd;
-   char *config_directive;
-   char *descript;
+   const char *config_directive;
+   const char *descript;
    char type;
    int *data;
 } cmd_table[] = {
