@@ -61,6 +61,7 @@ static const UPSDRIVER drivers[] = {
      dumb_ups_setup,
      dumb_ups_close,
      dumb_ups_kill_power,
+     NULL,
      dumb_ups_read_static_data,
      dumb_ups_read_volatile_data,
      dumb_ups_get_capabilities,
@@ -75,6 +76,7 @@ static const UPSDRIVER drivers[] = {
      apcsmart_ups_setup,
      apcsmart_ups_close,
      apcsmart_ups_kill_power,
+     apcsmart_ups_shutdown,
      apcsmart_ups_read_static_data,
      apcsmart_ups_read_volatile_data,
      apcsmart_ups_get_capabilities,
@@ -89,6 +91,7 @@ static const UPSDRIVER drivers[] = {
      net_ups_setup,
      net_ups_close,
      net_ups_kill_power,
+     NULL,
      net_ups_read_static_data,
      net_ups_read_volatile_data,
      net_ups_get_capabilities,
@@ -103,6 +106,7 @@ static const UPSDRIVER drivers[] = {
      usb_ups_setup,
      usb_ups_close,
      usb_ups_kill_power,
+     usb_ups_shutdown,
      usb_ups_read_static_data,
      usb_ups_read_volatile_data,
      usb_ups_get_capabilities,
@@ -117,6 +121,7 @@ static const UPSDRIVER drivers[] = {
      snmp_ups_setup,
      snmp_ups_close,
      snmp_ups_kill_power,
+     NULL,
      snmp_ups_read_static_data,
      snmp_ups_read_volatile_data,
      snmp_ups_get_capabilities,
@@ -131,6 +136,7 @@ static const UPSDRIVER drivers[] = {
      test_ups_setup,
      test_ups_close,
      test_ups_kill_power,
+     NULL,
      test_ups_read_static_data,
      test_ups_read_volatile_data,
      test_ups_get_capabilities,
@@ -145,6 +151,7 @@ static const UPSDRIVER drivers[] = {
      pcnet_ups_setup,
      pcnet_ups_close,
      pcnet_ups_kill_power,
+     NULL,
      pcnet_ups_read_static_data,
      pcnet_ups_read_volatile_data,
      pcnet_ups_get_capabilities,
@@ -166,6 +173,7 @@ static const UPSDRIVER drivers[] = {
      NULL,
      NULL,
      NULL,
+     NULL,
      NULL }
 };
 
