@@ -164,9 +164,7 @@ static const PAIRS table[] = {
    {"NISPORT",   match_int,   WHERE(statusport), 0},
 
    /* Configuration parameters for event logging */
-   {"EVENTFILE",     match_str, WHERE(eventfile),    SIZE(eventfile)},
    {"EVENTSFILE",    match_str, WHERE(eventfile),    SIZE(eventfile)},
-   {"EVENTFILEMAX",  match_int, WHERE(eventfilemax), 0},
    {"EVENTSFILEMAX", match_int, WHERE(eventfilemax), 0},
 
    /* Configuration parameters to control system logging */
@@ -567,9 +565,6 @@ static int ParseConfig(UPSINFO *ups, char *line)
  */
 void init_ups_struct(UPSINFO *ups)
 {
-   /* put some basic information for sanity checks */
-   astrncpy(ups->release, APCUPSD_RELEASE, sizeof(ups->release));
-
    ups->fd = -1;
 
    ups->set_plugged();
